<?php
/**
*
* @package acp
* @version $Id: acp_spammers.php 1373 2009-02-12 08:55:19Z xhotshotx $
* @copyright (c) 2009 Michael Williams (mtotheikle) startrekguide.com
* @license http://opensource.org/licenses/gpl-license.php GNU Public License
*
*/

/**
* @ignore
*/
if (!defined('IN_PHPBB'))
{
	exit;
}

/**
* @package acp
*/
class acp_spammers
{
	public $u_action;

	private $new_config = array();
	private $form_key;
	private $submit;
	private $action;

	function main($id, $mode)
	{
		global $db, $user, $auth, $template;
		global $config, $phpbb_root_path, $phpbb_admin_path, $phpEx;

		$user->add_lang('mods/spammers');

		$this->action	= request_var('action', '');
		$this->submit = (isset($_POST['submit'])) ? true : false;

		$this->form_key = 'acp_spammer_mod';
		add_form_key($this->form_key);

		/**
		*	Validation types are:
		*		string, int, bool,
		*		script_path (absolute path in url - beginning with / and no trailing slash),
		*		rpath (relative), rwpath (realtive, writable), path (relative path, but able to escape the root), wpath (writable)
		*/
		switch ($mode)
		{
			case 'settings':
			default:
				$this->settings();
			break;
		}

	}

	/**
	 * Select list of groups
	 *
	 * @param int $default
	 * @return option list of groups
	 */
	function group_select($default = 0)
	{
		global $db, $user;

		$sql_where = ($user->data['user_type'] == USER_FOUNDER) ? '' : 'WHERE group_founder_manage = 0';
		$sql = 'SELECT group_id, group_name, group_type
				FROM ' . GROUPS_TABLE . "
				$sql_where
				ORDER BY group_name";
		$result = $db->sql_query($sql);

		$group_options = '';
		while ($row = $db->sql_fetchrow($result))
		{
			$selected = ($row['group_id'] == $default) ? ' selected="selected"' : '';
			$group_name = ($row['group_type'] == GROUP_SPECIAL) ? $user->lang['G_' . $row['group_name']] : $row['group_name'];
			$group_options .= '<option value="' . $row['group_id'] . '"' . $selected . '>' . $group_name . '</option>';
		}
		$db->sql_freeresult($result);

		return $group_options;
	}

	/**
	 * General Settings.
	 *
	 */
	private function settings()
	{
		global $config, $user, $template;

		$display_vars = array(
			'title'	=> 'ACP_SPAMMERS',
			'vars'	=> array(
				'legend1'					=> 'ACP_SPAMMER_SETTINGS',
				'spammer_detection_enable'	=> array('lang' => 'SPAMMER_MOD_ENABLED',		'validate' => 'bool',	'type' => 'radio:yes_no', 'explain' => true),
				'spammer_notify_pm'			=> array('lang'	=> 'SPAMMER_SEND_PM',			'validate' => 'bool', 	'type' => 'radio:yes_no', 'explain' => true),
				'spammer_pm_group'			=> array('lang'	=> 'SPAMMER_PM_GROUP', 			'validate' => 'int', 	'type' => 'select', 'method' => 'group_select', 'params' => array('{CONFIG_VALUE}', false), 'explain' => true),
				'spammer_notify_post'		=> array('lang' => 'SPAMMER_NOTIFY_POST',		'validate' => 'bool', 	'type' => 'radio:yes_no', 'explain' => true),
				'spammer_poster_id'			=> array('lang' => 'SPAMMER_POSTER_ID', 		'validate' => 'int', 	'type' => 'text:3:8', 'explain' => true),
				'spammer_forum_id'			=> array('lang' => 'SPAMMER_FORUM_ID', 			'validate' => 'int', 	'type' => 'select', 'function' => 'make_forum_select', 'params' => array('{CONFIG_VALUE}', false, false, true, true, true), 'explain' => true),
				'spammer_topic_title'		=> array('lang' => 'SPAMMER_TOPIC_TITLE', 		'validate' => 'string',	'type' => 'text:40:255', 'explain' => true),

				'legend2'					=> 'ACP_BOT_SCOUT',
				'bot_scout_api'				=> array('lang' => 'BOT_SCOUT_API',				'validate' => 'string',	'type' => 'text:40:255', 'explain' => true),
				'bot_scout_enabled'			=> array('lang' => 'BOT_SCOUT_ENABLED',			'validate' => 'bool',	'type' => 'radio:yes_no', 'explain' => true),

				'legend3'					=> 'ACP_STOP_FORUM_SPAM',
				'stop_forum_api'			=> array('lang' => 'STOP_FORUM_API',			'validate' => 'string',	'type' => 'text:40:255', 'explain' => true),
				'stop_forum_enabled'		=> array('lang' => 'STOP_FORUM_ENABLED',		'validate' => 'bool',	'type' => 'radio:yes_no', 'explain' => true),

			)
		);


		if (isset($display_vars['lang']))
		{
			$user->add_lang($display_vars['lang']);
		}

		$this->new_config = $config;
		$cfg_array = (isset($_REQUEST['config'])) ? utf8_normalize_nfc(request_var('config', array('' => ''), true)) : $this->new_config;
		$error = array();

		// We validate the complete config if whished
		validate_config_vars($display_vars['vars'], $cfg_array, $error);

		if ($this->submit && !check_form_key($this->form_key))
		{
			$error[] = $user->lang['FORM_INVALID'];
		}
		// Do not write values if there is an error
		if (sizeof($error))
		{
			$this->submit = false;
		}

		// We go through the display_vars to make sure no one is trying to set variables he/she is not allowed to...
		foreach ($display_vars['vars'] as $config_name => $null)
		{
			if (!isset($cfg_array[$config_name]) || strpos($config_name, 'legend') !== false)
			{
				continue;
			}

			$this->new_config[$config_name] = $config_value = $cfg_array[$config_name];

			if ($this->submit)
			{
				set_config($config_name, $config_value);
			}
		}

		if ($this->submit)
		{
			trigger_error($user->lang['SPAMMER_CONFIG_UPDATED'] . adm_back_link($this->u_action));
		}

		$this->tpl_name = 'acp_board';
		$this->page_title = $display_vars['title'];

		$template->assign_vars(array(
			'L_TITLE'			=> $user->lang[$display_vars['title']],
			'L_TITLE_EXPLAIN'	=> $user->lang[$display_vars['title'] . '_EXPLAIN'],

			'S_ERROR'			=> (sizeof($error)) ? true : false,
			'ERROR_MSG'			=> implode('<br />', $error),

			'U_ACTION'			=> $this->u_action)
		);

		// Output relevant page
		foreach ($display_vars['vars'] as $config_key => $vars)
		{
			if (!is_array($vars) && strpos($config_key, 'legend') === false)
			{
				continue;
			}

			if (strpos($config_key, 'legend') !== false)
			{
				$template->assign_block_vars('options', array(
					'S_LEGEND'		=> true,
					'LEGEND'		=> (isset($user->lang[$vars])) ? $user->lang[$vars] : $vars)
				);

				continue;
			}

			$type = explode(':', $vars['type']);

			$l_explain = '';
			if ($vars['explain'] && isset($vars['lang_explain']))
			{
				$l_explain = (isset($user->lang[$vars['lang_explain']])) ? $user->lang[$vars['lang_explain']] : $vars['lang_explain'];
			}
			else if ($vars['explain'])
			{
				$l_explain = (isset($user->lang[$vars['lang'] . '_EXPLAIN'])) ? $user->lang[$vars['lang'] . '_EXPLAIN'] : '';
			}

			$content = build_cfg_template($type, $config_key, $this->new_config, $config_key, $vars);

			if (empty($content))
			{
				continue;
			}

			$template->assign_block_vars('options', array(
				'KEY'			=> $config_key,
				'TITLE'			=> (isset($user->lang[$vars['lang']])) ? $user->lang[$vars['lang']] : $vars['lang'],
				'S_EXPLAIN'		=> $vars['explain'],
				'TITLE_EXPLAIN'	=> $l_explain,
				'CONTENT'		=> $content,
				)
			);

			unset($display_vars['vars'][$config_key]);
		}
	}
}
?>