<?php
/**
*
* @package Spammer Dection
* @version $Id: spammer_base.php 1373 2009-02-12 08:55:19Z xhotshotx $
* @copyright (c) 2009 Michael Williams (mtotheikle) http://startrekguide.com
* @license http://opensource.org/licenses/gpl-license.php GNU Public License
*
* Special thanks to Francis Lewis (Handyman) for some ideas on handling data with simple XML.
*/

/**
* @ignore
*/
if (!defined('IN_PHPBB'))
{
	exit;
}

class spammer_base
{
	/**
	 * Simple XML Element Object
	 *
	 * @var object
	 */
	public static $xml;

	/**
	 * Holds are error message.
	 *
	 * @var string
	 */
	public static $error;

	/**
	 * Holds the data that should be validate at the spammer database site.
	 *
	 * @var array
	 */
	public static $validate = array();

	/**
	 * Classes for checking user data.
	 * To add a new method just create a new array with: lang, link, and class then create a new class inside root/includes/spammers.
	 */
	protected static $spam_sites = array(
		2		=> array(
			'lang'		=> 'BOT_SCOUT',
			'link'		=> 'http://www.botscout.com/',
			'class'		=> 'bot_scout',
		),
		3		=> array(
			'lang'		=> 'STOP_FORUM_SPAM',
			'link'		=> 'http://www.stopforumspam.com/',
			'class'		=> 'stop_forum_spam',
		),
	);

	/**
	 * Holds all the people that will receive a PM warning of a spammer.
	 *
	 * @var array
	 */
	private static $address_list = array();

	public static function init($user_data)
	{
		global $user, $config, $phpbb_root_path, $phpEx;

		if (!$config['spammer_detection_enable'])
		{
			return;
		}

		$user->add_lang('mods/spammers');

		// Check all sites for spammer. Let the site class decide if it should check the site or not.
		foreach (self::$spam_sites as $site)
		{
			if (file_exists($phpbb_root_path . 'includes/spammers/' . $site['class'] . '.' . $phpEx))
			{
				include $phpbb_root_path . 'includes/spammers/' . $site['class'] . '.' . $phpEx;

				call_user_func(array($site['class'], 'check_spammer'), $user_data);
			}
		}

		// Check if any sites returned data.
		if (sizeof(self::$error))
		{
			if ($config['spammer_notify_pm'])
			{
				// Let users with ban permissions know about our little spammer.
				self::send_pm();
			}

			if ($config['spammer_notify_post'])
			{
				self::notify_post();
			}
		}
	}

	/**
	 * Sets self::$xml to simple xml object.
	 *
	 * @param string $site site to parse
	 * @param array $validate data that should be validated
	 */
	protected static function load_file($site, $validate)
	{
		$values = array();

		foreach ($validate as $key => $value)
		{
			$encoded = urlencode(stripslashes($value));
			$values[] = $key . '=' . $encoded;
		}

		// Implode the array into a url string.
		$params = implode('&', $values);

		self::$xml = @simplexml_load_file($site . $params);
	}

	protected static function notify_post()
	{
		global $phpbb_root_path, $phpEx, $user, $auth, $db, $config;

		$forum_id = $config['spammer_forum_id'];

		if (!$forum_id)
		{
			// Cant continue without this.
			return;
		}

		// No message? Cant send.
		if (self::$error == '' || empty(self::$error))
		{
			return;
		}

		// Backup our current user.
		$backup = array(
			'user'	=> $user,
			'auth'	=> $auth,
		);

		// Get our user that we need to post as.
		$sql = 'SELECT *
			FROM ' . USERS_TABLE . '
			WHERE user_id = ' . (int) $config['spammer_poster_id'];
		$result = $db->sql_query($sql);
		$row = $db->sql_fetchrow($result);
		$db->sql_freeresult($result);

		if (!$row)
		{
			// No user!
			return;
		}

		$user->data = array_merge($user->data, $row);

		// Set new auth.
		$auth->acl($user->data);

		// Get our forum name.
		$sql = 'SELECT forum_name
			FROM ' . FORUMS_TABLE . '
			WHERE forum_id = ' . $config['spammer_forum_id'];
		$result = $db->sql_query($sql);
		$forum_name = $db->sql_fetchfield('forum_name');
		$db->sql_freeresult($result);

		// Overwrite user I.P
		$user->ip = '0.0.0.0';

		// Setup message parser and parse our message.
		if (!class_exists('parse_message'))
		{
			include($phpbb_root_path . 'includes/message_parser.' . $phpEx);
		}
		include($phpbb_root_path . 'includes/functions_posting.' . $phpEx);

		$message_parser = new parse_message(self::$error);
		$message_parser->parse(true, true, true, true, true, true, true, true, 'post');

		// Replace topic title config place holders with values.
		$topic_title = str_replace('{USERNAME}', self::$validate['username'], $config['spammer_topic_title']);

		// Setup our post data.
		$post_data = array(
			'forum_id'			=> $config['spammer_forum_id'],
			'forum_name'		=> $forum_name,
			'topic_title'		=> $topic_title,
			'poster_id'			=> $user->data['user_id'],
			'post_approved'		=> true,
			'icon_id'			=> 0,
			'notify'			=> false,
			'notify_set'		=> 0,
			'post_time'			=> time(),
			'enable_indexing'	=> true,
			'message'			=> $message_parser->message,
			'message_md5'		=> md5($message_parser->message),
			'enable_bbcode'		=> true,
			'enable_smilies'	=> false,
			'enable_urls'		=> true,
			'enable_sig'		=> true,
			'bbcode_bitfield'	=> $message_parser->bbcode_bitfield,
			'bbcode_uid'		=> $message_parser->bbcode_uid,
			'post_edit_locked'	=> false,
		);

		$poll = array();
		submit_post('post', $topic_title, $user->data['username'], POST_NORMAL, $poll, $post_data);

		// Restore user data.
		extract($backup);

		// Back to the actual user
		$auth->acl($user->data);
	}

	protected static function send_pm()
	{
		global $phpbb_root_path, $phpEx, $user, $config;

		if (!class_exists('parse_message'))
		{
			include($phpbb_root_path . 'includes/message_parser.' . $phpEx);
			include($phpbb_root_path . 'includes/functions_privmsgs.' . $phpEx);
		}

		// Setup the PM message parser.
		$message_parser = new parse_message();
		$message_parser->message = self::$error;
		$message_parser->parse(true, true, true, true, true, true, true);

		// Setup the PM.
		$pm_data = array(
			'from_user_id'		=> 2,
			'from_username'		=> 'Spammer Bot',
			'address_list'		=> array('g' => array($config['spammer_pm_group'] => 'to')),
			'icon_id'			=> 10,
			'from_user_ip'		=> $user->ip,
			'enable_bbcode'		=> true,
			'enable_smilies'	=> true,
			'enable_urls'		=> true,
			'enable_sig'		=> false,
			'message'			=> $message_parser->message,
			'bbcode_bitfield'	=> $message_parser->bbcode_bitfield,
			'bbcode_uid'		=> $message_parser->bbcode_uid,
		);

		// Send the PM to the users.
		submit_pm('post', $user->lang['NEW_SPAMMER'], $pm_data, false);
	}
}
?>